//      (_||_/       Utils.cc
//      (    )       Fonctions divers
//     ( o  0 )
//-OOO--(_)---OOO---------------------------------------
//                   Copyright (C) 2006 By Dominique Roux-Serret 
// .OOOo      oOOO.  roux-serret@ifrance.com
//-(   )------(   )---------------------------------------
//  ( (        ) /   Le 03/01/2006
//   (_)      (_/

//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 or version 3 of the License.

//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

//    You should have received a copy of the GNU General Public License along
//    with this program; if not, write to the Free Software Foundation, Inc.,
//    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zlib.h>

#ifdef WINDOWS
#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#endif

#include "utils.h"
#include "preference.h"
#include "sprite.h"

/*** Variables globales ***/
/**************************/
extern sPreference Pref;
#if defined(LINUX_MODE) || defined(CAANOO_MODE)
extern char DefPath[]; // Chemin par defaut dans arg
#endif

/*** Dfinition gnrals ***/
/***************************/
#ifdef MAC_OSX
#define MAC_LINUX
#endif

#ifdef LINUX_MODE
#define MAC_LINUX
#endif

//ZX: gz stuff
size_t 
my_gzwrite(const void *ptr, size_t size, size_t nmemb, gzFile stream)
{
  size_t num_bytes = gzwrite( stream, ptr, size * nmemb );
  return num_bytes / size;
}

size_t 
my_gzread(void *ptr, size_t size, size_t nmemb, gzFile stream)
{
  size_t num_bytes = gzread( stream, ptr, size * nmemb );
  return num_bytes / size;
}


/*** Test si un fichier exite ***/
/********************************/
bool FileExiste(const char *Path)
{
  FILE* File = fopen(Path, "r");
  if (File) {
    fclose(File);
    return true;
  }
  std::string s_Path( Path );
  s_Path += ".gz";
  gzFile file=gzopen(s_Path.c_str(),"r");

  if(file==NULL)
    return false;

  gzclose(file);
  return true;
}


/*** Charge un fichier en Mmoire ***/
/************************************/
#ifdef MAC_LINUX
// Version linux
long ChargeFichier(const char *Path,unsigned char *&Buf)
{
  gzFile  file;
  std::string s_Path( Path );
  s_Path += ".gz";

  file=gzopen(s_Path.c_str(),"r");
  if(!file) {
    cerr <<"ERREUR: Impossible d'ouvrir '"<<s_Path<<"'"<<endl;
    perror("gzopen");
    return -1;
  }
  
  if(gzseek(file,0L,SEEK_END)==0) {
    perror("gzseek");
    gzclose(file);
    return -1;
  }

  //ZX: compute size
  char Buffer[512];
  long L = 0;
  while (! gzeof(file)) {
    L += my_gzread( Buffer, 1, 512, file);
  }
  gzseek(file,0L,SEEK_SET);

  Buf=new unsigned char [L+1];
  if(Buf==NULL) {
    cerr <<"ERREUR: Memoire insuffisante!"<<endl;
    gzclose(file);
    return -1;
  }

  long Compt=L;
  unsigned char *Po=Buf;

  while(Compt>1024) {
    AfficheChargeur();
    if( my_gzread(Po,1,1024,file) != 1024 ) {
      cerr <<"ERREUR de lecture du fichier '"<<Path<<"'"<<endl;
      perror("gzread");
      gzclose(file);
      delete [] Buf;
      return -1;
    }
    Compt-=1024;
    Po+=1024;
  }
  
  if(Compt) { // Ne fait pas le test  cause d'un bug dans windows
    my_gzread(Po,1,(unsigned int)Compt,file);
  }
  
  gzclose(file);
  return L;
}
#endif

/*** Sauve un Fichier ***/
/************************/
#ifdef MAC_LINUX
// Version linux
bool SauveFichier(const char *Path,char *Buf,long L)
{
  gzFile  file;
  std::string s_Path( Path );
  s_Path += ".gz";
  
  file=gzopen(s_Path.c_str(),"w");
  if(!file) {
    cerr <<"ERREUR: Impossible d'ouvrir '"<<s_Path<<"'"<<endl;
    perror("gzopen");
    return false;
  }
  
  while(L>512) {
    if( my_gzwrite(Buf,1,512,file) != 512 ) {
      cerr <<"ERREUR d'ecriture du fichier '"<<s_Path<<"'"<<endl;
      perror("gzwrite");
      gzclose(file);
      return false;
    }
    L-=512;
    Buf+=512;
  }

  if(L>0) {
    if( my_gzwrite(Buf,1,(size_t)L,file) != (size_t)L ) {
      cerr <<"ERREUR d'ecriture du fichier '"<<Path<<"'"<<endl;
      perror("gzwrite");
      gzclose(file);
      return false;
    }
  }
  
  gzclose(file);
  return true;
}
#endif

#ifdef WINDOWS
// Version windows
bool SauveFichier(const char *Path,char *Buf,long L)
{
  HFILE file;
  int Lec;

  file=_lcreat(Path,0);
  if(!file) {
    cerr <<"ERREUR: Impossible de crer le fichier '"<<Path<<"'"<<endl;
    return false;
  }
  
  Lec=_hwrite(file,Buf,L);
  _lclose(file);

  if(Lec!=L) {
    cerr <<"Problme d'ecriture du fichier '"<<Path<<"' ecris="<<Lec<<" au lieux de ="<<L<<endl;
    return false;
  }

  return true;
}
#endif

/*** Met le bon chemin pour charger un fichier ***/
/*************************************************/
#if defined(LINUX_MODE) && !defined(__AMIGAOS4__)
// Version Linux
void GetPath(char *Path)
{
  char Provi[512];

  strcpy(Provi,Path);

  if(DefPath[0]) {
    sprintf(Path,"%s%s",DefPath,Provi);
    if(FileExiste(Path)) return;
  }

# ifndef DATA_DIR
  sprintf(Path,"./%s",Provi);
# else
  sprintf(Path,"%s/%s",DATA_DIR,Provi);
# endif
  if(FileExiste(Path)) return;
  sprintf(Path,"./%s",Provi);
  if(FileExiste(Path)) return;
  sprintf(Path,"/usr/local/share/Ri-li/%s",Provi);
  if(FileExiste(Path)) return;
  sprintf(Path,"/usr/share/Ri-li/%s",Provi);
  if(FileExiste(Path)) return;
  sprintf(Path,"/usr/share/games/Ri-li/%s",Provi);
  if(FileExiste(Path)) return;
  
  cerr <<"Impossible de trouver le fichier '"<<Provi<<endl;
  exit(-1);
}
#endif

#ifdef __AMIGAOS4__
// Version AmigaOS4
void GetPath(char *Path)
{
  char Provi[512];

  strcpy(Provi,Path);

  sprintf(Path,"PROGDIR:%s",Provi);
  if(FileExiste(Path)) return;
  
  cerr <<"Impossible de trouver le fichier '"<<Path<<endl;
  exit(-1);
}
#endif

#ifdef MAC_OSX
// Version Mac OSX
void GetPath(char *Path)
{
  char Provi[512];

  strcpy(Provi,Path);

  sprintf(Path,"Ri-li.app/Contents/Resources/%s",Provi);
  if(FileExiste(Path)) return;
  
  cerr <<"Impossible de trouver le fichier '"<<Path<<endl;
  exit(-1);
}
#endif

#ifdef WINDOWS
//  Version Windows , chemin directe
void GetPath(char *Path)
{ }
#endif

/*** Charge les prferences ***/
/******************************/
bool LoadPref(void)
{
  int L;
  unsigned char *Provi;
  
#if defined(PND_MODE) || defined(CAANOO_MODE)
  char PathPref[128];
  strcpy(PathPref, "./ri-li.pref");
#elif defined(LINUX_MODE) && !defined(__AMIGAOS4__)
  char PathPref[512];
  char *Env=getenv("HOME");
  sprintf(PathPref,"%s/.ri-li.pref",Env);
#elif defined(__AMIGAOS4__)
  char PathPref[512];
  char *Env="PROGDIR:";
  sprintf(PathPref,"%s.ri-li.pref",Env);
#elif MAC_OSX
  char PathPref[512];
  char *Env=getenv("HOME");
  sprintf(PathPref,"%s/Library/Preferences/Ri-li.pref",Env);
#endif

#ifdef WINDOWS
  // Version Windows, Chemin directe
  char *PathPref="ri-li.pref";
#endif
  
  if(FileExiste(PathPref)) {
    L=ChargeFichier(PathPref,Provi);
    if(L>0) {
      memcpy((char*)&Pref,Provi,L);
      delete [] Provi;
      return true;
    }
  }
  
  return false;
}

/*** Sauve les preferences ***/
/*****************************/
void SauvePref(void)
{  
#if defined(PND_MODE) || defined(CAANOO_MODE)
  char Provi[128];
  strcpy(Provi, "./ri-li.pref");
  SauveFichier(Provi,(char*)&Pref,sizeof(sPreference));
#elif defined(LINUX_MODE) && !defined(__AMIGAOS4__)
  char Provi[512];
  char *Env=getenv("HOME");
  sprintf(Provi,"%s/.ri-li.pref",Env);
  SauveFichier(Provi,(char*)&Pref,sizeof(sPreference));
#elif defined(__AMIGAOS4__)
  char Provi[512];
  char *Env="PROGDIR:";
  sprintf(Provi,"%s.ri-li.pref",Env);
  SauveFichier(Provi,(char*)&Pref,sizeof(sPreference));
#endif

#ifdef MAC_OSX
  char Provi[512];
  char *Env=getenv("HOME");
  sprintf(Provi,"%s/Library/Preferences/Ri-li.pref",Env);
  SauveFichier(Provi,(char*)&Pref,sizeof(sPreference));
#endif

#ifdef WINDOWS
  // Version Windows, chemin directe
  SauveFichier("ri-li.pref",(char*)&Pref,sizeof(sPreference));
#endif
}

